"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Event = void 0;
const eventFetcher_1 = require("../eventFetcher");
const chainSlugMap_1 = require("../utils/chainSlugMap");
const promiseQueue_1 = require("../promiseQueue");
class Event {
    constructor(provider, chainId, batchBlocks, address) {
        if (!provider) {
            throw new Error('expected provider');
        }
        this.provider = provider;
        this.chainId = chainId;
        this.batchBlocks = batchBlocks;
        this.address = address;
    }
    async _getEvents(filter, fromBlock, toBlock) {
        const eventFetcher = new eventFetcher_1.EventFetcher({
            provider: this.provider,
            batchBlocks: this.batchBlocks
        });
        if (!toBlock) {
            toBlock = await this.provider.getBlockNumber();
        }
        const events = await eventFetcher.fetchEvents([filter], { fromBlock, toBlock });
        console.log(`populating events. count: ${events.length}`);
        return await this.populateEvents(events);
    }
    async populateEvents(events) {
        events = events.map(x => this.toTypedEvent(x));
        const promiseFns = events.map((event) => () => this.addContextToEvent(event, this.chainId));
        const populatedEvents = [];
        await (0, promiseQueue_1.promiseQueue)(promiseFns, async (fn) => {
            populatedEvents.push(await fn());
        }, { concurrency: 20 });
        return populatedEvents;
    }
    toTypedEvent(ethersEvent) {
        throw new Error('Not implemented');
    }
    async addContextToEvent(event, chainId) {
        const context = await this.getEventContext(event.eventLog, chainId);
        event.context = context;
        return event;
    }
    async getEventContext(event, chainId) {
        try {
            const chainSlug = this.getChainSlug(chainId);
            const transactionHash = event.transactionHash;
            const transactionIndex = event.transactionIndex;
            const logIndex = event.logIndex;
            const blockNumber = event.blockNumber;
            const [{ timestamp: blockTimestamp }, { value, nonce, gasLimit, gasPrice, data }, { from, to, gasUsed }] = await Promise.all([
                this.provider.getBlock(blockNumber),
                this.provider.getTransaction(transactionHash),
                this.provider.getTransactionReceipt(transactionHash)
            ]);
            return {
                chainSlug,
                chainId,
                transactionHash,
                transactionIndex,
                logIndex,
                blockNumber,
                blockTimestamp,
                from,
                to,
                value: value.toString(),
                nonce: Number(nonce.toString()),
                gasLimit: Number(gasLimit === null || gasLimit === void 0 ? void 0 : gasLimit.toString()),
                gasUsed: Number(gasUsed === null || gasUsed === void 0 ? void 0 : gasUsed.toString()),
                gasPrice: gasPrice === null || gasPrice === void 0 ? void 0 : gasPrice.toString(),
                data
            };
        }
        catch (err) {
            console.log('getEventContext error:', err, chainId, event);
            throw err;
        }
    }
    getChainSlug(chainId) {
        const chainSlug = chainSlugMap_1.chainSlugMap[chainId];
        if (!chainSlug) {
            throw new Error(`Invalid chain: ${chainId}`);
        }
        return chainSlug;
    }
}
exports.Event = Event;
